/****************************************************************************
** Meta object code from reading C++ file 'qwebsocket.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtwebsockets/src/websockets/qwebsocket.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qwebsocket.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10QWebSocketE_t {};
} // unnamed namespace

template <> constexpr inline auto QWebSocket::qt_create_metaobjectdata<qt_meta_tag_ZN10QWebSocketE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QWebSocket",
        "aboutToClose",
        "",
        "connected",
        "disconnected",
        "stateChanged",
        "QAbstractSocket::SocketState",
        "state",
        "proxyAuthenticationRequired",
        "QNetworkProxy",
        "proxy",
        "QAuthenticator*",
        "pAuthenticator",
        "authenticationRequired",
        "authenticator",
        "readChannelFinished",
        "textFrameReceived",
        "frame",
        "isLastFrame",
        "binaryFrameReceived",
        "textMessageReceived",
        "message",
        "binaryMessageReceived",
        "error",
        "QAbstractSocket::SocketError",
        "errorOccurred",
        "pong",
        "elapsedTime",
        "payload",
        "bytesWritten",
        "bytes",
        "peerVerifyError",
        "QSslError",
        "sslErrors",
        "QList<QSslError>",
        "errors",
        "preSharedKeyAuthenticationRequired",
        "QSslPreSharedKeyAuthenticator*",
        "alertSent",
        "QSsl::AlertLevel",
        "level",
        "QSsl::AlertType",
        "type",
        "description",
        "alertReceived",
        "handshakeInterruptedOnError",
        "close",
        "QWebSocketProtocol::CloseCode",
        "closeCode",
        "reason",
        "open",
        "QUrl",
        "url",
        "QNetworkRequest",
        "request",
        "QWebSocketHandshakeOptions",
        "options",
        "ping",
        "ignoreSslErrors"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'aboutToClose'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'connected'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'disconnected'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'stateChanged'
        QtMocHelpers::SignalData<void(QAbstractSocket::SocketState)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'proxyAuthenticationRequired'
        QtMocHelpers::SignalData<void(const QNetworkProxy &, QAuthenticator *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { 0x80000000 | 11, 12 },
        }}),
        // Signal 'authenticationRequired'
        QtMocHelpers::SignalData<void(QAuthenticator *)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 14 },
        }}),
        // Signal 'readChannelFinished'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'textFrameReceived'
        QtMocHelpers::SignalData<void(const QString &, bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 17 }, { QMetaType::Bool, 18 },
        }}),
        // Signal 'binaryFrameReceived'
        QtMocHelpers::SignalData<void(const QByteArray &, bool)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 17 }, { QMetaType::Bool, 18 },
        }}),
        // Signal 'textMessageReceived'
        QtMocHelpers::SignalData<void(const QString &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 21 },
        }}),
        // Signal 'binaryMessageReceived'
        QtMocHelpers::SignalData<void(const QByteArray &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 21 },
        }}),
        // Signal 'error'
        QtMocHelpers::SignalData<void(QAbstractSocket::SocketError)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 23 },
        }}),
        // Signal 'errorOccurred'
        QtMocHelpers::SignalData<void(QAbstractSocket::SocketError)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 23 },
        }}),
        // Signal 'pong'
        QtMocHelpers::SignalData<void(quint64, const QByteArray &)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::ULongLong, 27 }, { QMetaType::QByteArray, 28 },
        }}),
        // Signal 'bytesWritten'
        QtMocHelpers::SignalData<void(qint64)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 30 },
        }}),
        // Signal 'peerVerifyError'
        QtMocHelpers::SignalData<void(const QSslError &)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 32, 23 },
        }}),
        // Signal 'sslErrors'
        QtMocHelpers::SignalData<void(const QList<QSslError> &)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 34, 35 },
        }}),
        // Signal 'preSharedKeyAuthenticationRequired'
        QtMocHelpers::SignalData<void(QSslPreSharedKeyAuthenticator *)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 37, 14 },
        }}),
        // Signal 'alertSent'
        QtMocHelpers::SignalData<void(QSsl::AlertLevel, QSsl::AlertType, const QString &)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 39, 40 }, { 0x80000000 | 41, 42 }, { QMetaType::QString, 43 },
        }}),
        // Signal 'alertReceived'
        QtMocHelpers::SignalData<void(QSsl::AlertLevel, QSsl::AlertType, const QString &)>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 39, 40 }, { 0x80000000 | 41, 42 }, { QMetaType::QString, 43 },
        }}),
        // Signal 'handshakeInterruptedOnError'
        QtMocHelpers::SignalData<void(const QSslError &)>(45, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 32, 23 },
        }}),
        // Slot 'close'
        QtMocHelpers::SlotData<void(QWebSocketProtocol::CloseCode, const QString &)>(46, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 47, 48 }, { QMetaType::QString, 49 },
        }}),
        // Slot 'close'
        QtMocHelpers::SlotData<void(QWebSocketProtocol::CloseCode)>(46, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 47, 48 },
        }}),
        // Slot 'close'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'open'
        QtMocHelpers::SlotData<void(const QUrl &)>(50, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 51, 52 },
        }}),
        // Slot 'open'
        QtMocHelpers::SlotData<void(const QNetworkRequest &)>(50, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 53, 54 },
        }}),
        // Slot 'open'
        QtMocHelpers::SlotData<void(const QUrl &, const QWebSocketHandshakeOptions &)>(50, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 51, 52 }, { 0x80000000 | 55, 56 },
        }}),
        // Slot 'open'
        QtMocHelpers::SlotData<void(const QNetworkRequest &, const QWebSocketHandshakeOptions &)>(50, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 53, 54 }, { 0x80000000 | 55, 56 },
        }}),
        // Slot 'ping'
        QtMocHelpers::SlotData<void(const QByteArray &)>(57, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 28 },
        }}),
        // Slot 'ping'
        QtMocHelpers::SlotData<void()>(57, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'ignoreSslErrors'
        QtMocHelpers::SlotData<void()>(58, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QWebSocket, qt_meta_tag_ZN10QWebSocketE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QWebSocket::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10QWebSocketE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10QWebSocketE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10QWebSocketE_t>.metaTypes,
    nullptr
} };

void QWebSocket::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QWebSocket *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->aboutToClose(); break;
        case 1: _t->connected(); break;
        case 2: _t->disconnected(); break;
        case 3: _t->stateChanged((*reinterpret_cast<std::add_pointer_t<QAbstractSocket::SocketState>>(_a[1]))); break;
        case 4: _t->proxyAuthenticationRequired((*reinterpret_cast<std::add_pointer_t<QNetworkProxy>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QAuthenticator*>>(_a[2]))); break;
        case 5: _t->authenticationRequired((*reinterpret_cast<std::add_pointer_t<QAuthenticator*>>(_a[1]))); break;
        case 6: _t->readChannelFinished(); break;
        case 7: _t->textFrameReceived((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 8: _t->binaryFrameReceived((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 9: _t->textMessageReceived((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->binaryMessageReceived((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 11: _t->error((*reinterpret_cast<std::add_pointer_t<QAbstractSocket::SocketError>>(_a[1]))); break;
        case 12: _t->errorOccurred((*reinterpret_cast<std::add_pointer_t<QAbstractSocket::SocketError>>(_a[1]))); break;
        case 13: _t->pong((*reinterpret_cast<std::add_pointer_t<quint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 14: _t->bytesWritten((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1]))); break;
        case 15: _t->peerVerifyError((*reinterpret_cast<std::add_pointer_t<QSslError>>(_a[1]))); break;
        case 16: _t->sslErrors((*reinterpret_cast<std::add_pointer_t<QList<QSslError>>>(_a[1]))); break;
        case 17: _t->preSharedKeyAuthenticationRequired((*reinterpret_cast<std::add_pointer_t<QSslPreSharedKeyAuthenticator*>>(_a[1]))); break;
        case 18: _t->alertSent((*reinterpret_cast<std::add_pointer_t<QSsl::AlertLevel>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QSsl::AlertType>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 19: _t->alertReceived((*reinterpret_cast<std::add_pointer_t<QSsl::AlertLevel>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QSsl::AlertType>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 20: _t->handshakeInterruptedOnError((*reinterpret_cast<std::add_pointer_t<QSslError>>(_a[1]))); break;
        case 21: _t->close((*reinterpret_cast<std::add_pointer_t<QWebSocketProtocol::CloseCode>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 22: _t->close((*reinterpret_cast<std::add_pointer_t<QWebSocketProtocol::CloseCode>>(_a[1]))); break;
        case 23: _t->close(); break;
        case 24: _t->open((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 25: _t->open((*reinterpret_cast<std::add_pointer_t<QNetworkRequest>>(_a[1]))); break;
        case 26: _t->open((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QWebSocketHandshakeOptions>>(_a[2]))); break;
        case 27: _t->open((*reinterpret_cast<std::add_pointer_t<QNetworkRequest>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QWebSocketHandshakeOptions>>(_a[2]))); break;
        case 28: _t->ping((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 29: _t->ping(); break;
        case 30: _t->ignoreSslErrors(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstractSocket::SocketState >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QNetworkProxy >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstractSocket::SocketError >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstractSocket::SocketError >(); break;
            }
            break;
        case 16:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QSslError> >(); break;
            }
            break;
        case 25:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QNetworkRequest >(); break;
            }
            break;
        case 27:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QNetworkRequest >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QWebSocket::*)()>(_a, &QWebSocket::aboutToClose, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QWebSocket::*)()>(_a, &QWebSocket::connected, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QWebSocket::*)()>(_a, &QWebSocket::disconnected, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QWebSocket::*)(QAbstractSocket::SocketState )>(_a, &QWebSocket::stateChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QWebSocket::*)(const QNetworkProxy & , QAuthenticator * )>(_a, &QWebSocket::proxyAuthenticationRequired, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QWebSocket::*)(QAuthenticator * )>(_a, &QWebSocket::authenticationRequired, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QWebSocket::*)()>(_a, &QWebSocket::readChannelFinished, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QWebSocket::*)(const QString & , bool )>(_a, &QWebSocket::textFrameReceived, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QWebSocket::*)(const QByteArray & , bool )>(_a, &QWebSocket::binaryFrameReceived, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QWebSocket::*)(const QString & )>(_a, &QWebSocket::textMessageReceived, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QWebSocket::*)(const QByteArray & )>(_a, &QWebSocket::binaryMessageReceived, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (QWebSocket::*)(QAbstractSocket::SocketError )>(_a, &QWebSocket::error, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (QWebSocket::*)(QAbstractSocket::SocketError )>(_a, &QWebSocket::errorOccurred, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (QWebSocket::*)(quint64 , const QByteArray & )>(_a, &QWebSocket::pong, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (QWebSocket::*)(qint64 )>(_a, &QWebSocket::bytesWritten, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (QWebSocket::*)(const QSslError & )>(_a, &QWebSocket::peerVerifyError, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (QWebSocket::*)(const QList<QSslError> & )>(_a, &QWebSocket::sslErrors, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (QWebSocket::*)(QSslPreSharedKeyAuthenticator * )>(_a, &QWebSocket::preSharedKeyAuthenticationRequired, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (QWebSocket::*)(QSsl::AlertLevel , QSsl::AlertType , const QString & )>(_a, &QWebSocket::alertSent, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (QWebSocket::*)(QSsl::AlertLevel , QSsl::AlertType , const QString & )>(_a, &QWebSocket::alertReceived, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (QWebSocket::*)(const QSslError & )>(_a, &QWebSocket::handshakeInterruptedOnError, 20))
            return;
    }
}

const QMetaObject *QWebSocket::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QWebSocket::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10QWebSocketE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QWebSocket::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 31)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 31;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 31)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 31;
    }
    return _id;
}

// SIGNAL 0
void QWebSocket::aboutToClose()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QWebSocket::connected()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QWebSocket::disconnected()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QWebSocket::stateChanged(QAbstractSocket::SocketState _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QWebSocket::proxyAuthenticationRequired(const QNetworkProxy & _t1, QAuthenticator * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void QWebSocket::authenticationRequired(QAuthenticator * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void QWebSocket::readChannelFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QWebSocket::textFrameReceived(const QString & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2);
}

// SIGNAL 8
void QWebSocket::binaryFrameReceived(const QByteArray & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1, _t2);
}

// SIGNAL 9
void QWebSocket::textMessageReceived(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void QWebSocket::binaryMessageReceived(const QByteArray & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void QWebSocket::error(QAbstractSocket::SocketError _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void QWebSocket::errorOccurred(QAbstractSocket::SocketError _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void QWebSocket::pong(quint64 _t1, const QByteArray & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1, _t2);
}

// SIGNAL 14
void QWebSocket::bytesWritten(qint64 _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void QWebSocket::peerVerifyError(const QSslError & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void QWebSocket::sslErrors(const QList<QSslError> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void QWebSocket::preSharedKeyAuthenticationRequired(QSslPreSharedKeyAuthenticator * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}

// SIGNAL 18
void QWebSocket::alertSent(QSsl::AlertLevel _t1, QSsl::AlertType _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1, _t2, _t3);
}

// SIGNAL 19
void QWebSocket::alertReceived(QSsl::AlertLevel _t1, QSsl::AlertType _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 19, nullptr, _t1, _t2, _t3);
}

// SIGNAL 20
void QWebSocket::handshakeInterruptedOnError(const QSslError & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1);
}
QT_WARNING_POP
