/****************************************************************************
** Meta object code from reading C++ file 'qwebsocketserver.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtwebsockets/src/websockets/qwebsocketserver.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qwebsocketserver.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16QWebSocketServerE_t {};
} // unnamed namespace

template <> constexpr inline auto QWebSocketServer::qt_create_metaobjectdata<qt_meta_tag_ZN16QWebSocketServerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QWebSocketServer",
        "acceptError",
        "",
        "QAbstractSocket::SocketError",
        "socketError",
        "serverError",
        "QWebSocketProtocol::CloseCode",
        "closeCode",
        "originAuthenticationRequired",
        "QWebSocketCorsAuthenticator*",
        "pAuthenticator",
        "newConnection",
        "peerVerifyError",
        "QSslError",
        "error",
        "sslErrors",
        "QList<QSslError>",
        "errors",
        "sslErrorsOccurred",
        "QSslSocket*",
        "socket",
        "preSharedKeyAuthenticationRequired",
        "QSslPreSharedKeyAuthenticator*",
        "authenticator",
        "alertSent",
        "QSsl::AlertLevel",
        "level",
        "QSsl::AlertType",
        "type",
        "description",
        "alertReceived",
        "handshakeInterruptedOnError",
        "closed",
        "SslMode",
        "SecureMode",
        "NonSecureMode"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'acceptError'
        QtMocHelpers::SignalData<void(QAbstractSocket::SocketError)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'serverError'
        QtMocHelpers::SignalData<void(QWebSocketProtocol::CloseCode)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'originAuthenticationRequired'
        QtMocHelpers::SignalData<void(QWebSocketCorsAuthenticator *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'newConnection'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'peerVerifyError'
        QtMocHelpers::SignalData<void(const QSslError &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Signal 'sslErrors'
        QtMocHelpers::SignalData<void(const QList<QSslError> &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Signal 'sslErrorsOccurred'
        QtMocHelpers::SignalData<void(QSslSocket *, const QList<QSslError> &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 20 }, { 0x80000000 | 16, 17 },
        }}),
        // Signal 'preSharedKeyAuthenticationRequired'
        QtMocHelpers::SignalData<void(QSslPreSharedKeyAuthenticator *)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Signal 'alertSent'
        QtMocHelpers::SignalData<void(QSsl::AlertLevel, QSsl::AlertType, const QString &)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 25, 26 }, { 0x80000000 | 27, 28 }, { QMetaType::QString, 29 },
        }}),
        // Signal 'alertReceived'
        QtMocHelpers::SignalData<void(QSsl::AlertLevel, QSsl::AlertType, const QString &)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 25, 26 }, { 0x80000000 | 27, 28 }, { QMetaType::QString, 29 },
        }}),
        // Signal 'handshakeInterruptedOnError'
        QtMocHelpers::SignalData<void(const QSslError &)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Signal 'closed'
        QtMocHelpers::SignalData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'SslMode'
        QtMocHelpers::EnumData<enum SslMode>(33, 33, QMC::EnumFlags{}).add({
            {   34, SslMode::SecureMode },
            {   35, SslMode::NonSecureMode },
        }),
    };
    return QtMocHelpers::metaObjectData<QWebSocketServer, qt_meta_tag_ZN16QWebSocketServerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QWebSocketServer::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QWebSocketServerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QWebSocketServerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16QWebSocketServerE_t>.metaTypes,
    nullptr
} };

void QWebSocketServer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QWebSocketServer *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->acceptError((*reinterpret_cast<std::add_pointer_t<QAbstractSocket::SocketError>>(_a[1]))); break;
        case 1: _t->serverError((*reinterpret_cast<std::add_pointer_t<QWebSocketProtocol::CloseCode>>(_a[1]))); break;
        case 2: _t->originAuthenticationRequired((*reinterpret_cast<std::add_pointer_t<QWebSocketCorsAuthenticator*>>(_a[1]))); break;
        case 3: _t->newConnection(); break;
        case 4: _t->peerVerifyError((*reinterpret_cast<std::add_pointer_t<QSslError>>(_a[1]))); break;
        case 5: _t->sslErrors((*reinterpret_cast<std::add_pointer_t<QList<QSslError>>>(_a[1]))); break;
        case 6: _t->sslErrorsOccurred((*reinterpret_cast<std::add_pointer_t<QSslSocket*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<QSslError>>>(_a[2]))); break;
        case 7: _t->preSharedKeyAuthenticationRequired((*reinterpret_cast<std::add_pointer_t<QSslPreSharedKeyAuthenticator*>>(_a[1]))); break;
        case 8: _t->alertSent((*reinterpret_cast<std::add_pointer_t<QSsl::AlertLevel>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QSsl::AlertType>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 9: _t->alertReceived((*reinterpret_cast<std::add_pointer_t<QSsl::AlertLevel>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QSsl::AlertType>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 10: _t->handshakeInterruptedOnError((*reinterpret_cast<std::add_pointer_t<QSslError>>(_a[1]))); break;
        case 11: _t->closed(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstractSocket::SocketError >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QSslError> >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QSslError> >(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSslSocket* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QWebSocketServer::*)(QAbstractSocket::SocketError )>(_a, &QWebSocketServer::acceptError, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QWebSocketServer::*)(QWebSocketProtocol::CloseCode )>(_a, &QWebSocketServer::serverError, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QWebSocketServer::*)(QWebSocketCorsAuthenticator * )>(_a, &QWebSocketServer::originAuthenticationRequired, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QWebSocketServer::*)()>(_a, &QWebSocketServer::newConnection, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QWebSocketServer::*)(const QSslError & )>(_a, &QWebSocketServer::peerVerifyError, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QWebSocketServer::*)(const QList<QSslError> & )>(_a, &QWebSocketServer::sslErrors, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QWebSocketServer::*)(QSslSocket * , const QList<QSslError> & )>(_a, &QWebSocketServer::sslErrorsOccurred, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QWebSocketServer::*)(QSslPreSharedKeyAuthenticator * )>(_a, &QWebSocketServer::preSharedKeyAuthenticationRequired, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QWebSocketServer::*)(QSsl::AlertLevel , QSsl::AlertType , const QString & )>(_a, &QWebSocketServer::alertSent, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QWebSocketServer::*)(QSsl::AlertLevel , QSsl::AlertType , const QString & )>(_a, &QWebSocketServer::alertReceived, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QWebSocketServer::*)(const QSslError & )>(_a, &QWebSocketServer::handshakeInterruptedOnError, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (QWebSocketServer::*)()>(_a, &QWebSocketServer::closed, 11))
            return;
    }
}

const QMetaObject *QWebSocketServer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QWebSocketServer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QWebSocketServerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QWebSocketServer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void QWebSocketServer::acceptError(QAbstractSocket::SocketError _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QWebSocketServer::serverError(QWebSocketProtocol::CloseCode _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QWebSocketServer::originAuthenticationRequired(QWebSocketCorsAuthenticator * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QWebSocketServer::newConnection()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QWebSocketServer::peerVerifyError(const QSslError & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void QWebSocketServer::sslErrors(const QList<QSslError> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void QWebSocketServer::sslErrorsOccurred(QSslSocket * _t1, const QList<QSslError> & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}

// SIGNAL 7
void QWebSocketServer::preSharedKeyAuthenticationRequired(QSslPreSharedKeyAuthenticator * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void QWebSocketServer::alertSent(QSsl::AlertLevel _t1, QSsl::AlertType _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1, _t2, _t3);
}

// SIGNAL 9
void QWebSocketServer::alertReceived(QSsl::AlertLevel _t1, QSsl::AlertType _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1, _t2, _t3);
}

// SIGNAL 10
void QWebSocketServer::handshakeInterruptedOnError(const QSslError & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void QWebSocketServer::closed()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}
QT_WARNING_POP
