/****************************************************************************
** Meta object code from reading C++ file 'qwebsocketdataprocessor_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtwebsockets/src/websockets/qwebsocketdataprocessor_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qwebsocketdataprocessor_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN23QWebSocketDataProcessorE_t {};
} // unnamed namespace

template <> constexpr inline auto QWebSocketDataProcessor::qt_create_metaobjectdata<qt_meta_tag_ZN23QWebSocketDataProcessorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QWebSocketDataProcessor",
        "pingReceived",
        "",
        "data",
        "pongReceived",
        "closeReceived",
        "QWebSocketProtocol::CloseCode",
        "closeCode",
        "closeReason",
        "textFrameReceived",
        "frame",
        "lastFrame",
        "binaryFrameReceived",
        "textMessageReceived",
        "message",
        "binaryMessageReceived",
        "errorEncountered",
        "code",
        "description",
        "process",
        "QIODevice*",
        "pIoDevice",
        "clear"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'pingReceived'
        QtMocHelpers::SignalData<void(const QByteArray &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 3 },
        }}),
        // Signal 'pongReceived'
        QtMocHelpers::SignalData<void(const QByteArray &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 3 },
        }}),
        // Signal 'closeReceived'
        QtMocHelpers::SignalData<void(QWebSocketProtocol::CloseCode, const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { QMetaType::QString, 8 },
        }}),
        // Signal 'textFrameReceived'
        QtMocHelpers::SignalData<void(const QString &, bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 10 }, { QMetaType::Bool, 11 },
        }}),
        // Signal 'binaryFrameReceived'
        QtMocHelpers::SignalData<void(const QByteArray &, bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 10 }, { QMetaType::Bool, 11 },
        }}),
        // Signal 'textMessageReceived'
        QtMocHelpers::SignalData<void(const QString &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 14 },
        }}),
        // Signal 'binaryMessageReceived'
        QtMocHelpers::SignalData<void(const QByteArray &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 14 },
        }}),
        // Signal 'errorEncountered'
        QtMocHelpers::SignalData<void(QWebSocketProtocol::CloseCode, const QString &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 17 }, { QMetaType::QString, 18 },
        }}),
        // Slot 'process'
        QtMocHelpers::SlotData<bool(QIODevice *)>(19, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Slot 'clear'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QWebSocketDataProcessor, qt_meta_tag_ZN23QWebSocketDataProcessorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QWebSocketDataProcessor::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23QWebSocketDataProcessorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23QWebSocketDataProcessorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN23QWebSocketDataProcessorE_t>.metaTypes,
    nullptr
} };

void QWebSocketDataProcessor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QWebSocketDataProcessor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->pingReceived((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 1: _t->pongReceived((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 2: _t->closeReceived((*reinterpret_cast<std::add_pointer_t<QWebSocketProtocol::CloseCode>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 3: _t->textFrameReceived((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 4: _t->binaryFrameReceived((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 5: _t->textMessageReceived((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->binaryMessageReceived((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 7: _t->errorEncountered((*reinterpret_cast<std::add_pointer_t<QWebSocketProtocol::CloseCode>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 8: { bool _r = _t->process((*reinterpret_cast<std::add_pointer_t<QIODevice*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 9: _t->clear(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QWebSocketDataProcessor::*)(const QByteArray & )>(_a, &QWebSocketDataProcessor::pingReceived, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QWebSocketDataProcessor::*)(const QByteArray & )>(_a, &QWebSocketDataProcessor::pongReceived, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QWebSocketDataProcessor::*)(QWebSocketProtocol::CloseCode , const QString & )>(_a, &QWebSocketDataProcessor::closeReceived, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QWebSocketDataProcessor::*)(const QString & , bool )>(_a, &QWebSocketDataProcessor::textFrameReceived, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QWebSocketDataProcessor::*)(const QByteArray & , bool )>(_a, &QWebSocketDataProcessor::binaryFrameReceived, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QWebSocketDataProcessor::*)(const QString & )>(_a, &QWebSocketDataProcessor::textMessageReceived, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QWebSocketDataProcessor::*)(const QByteArray & )>(_a, &QWebSocketDataProcessor::binaryMessageReceived, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QWebSocketDataProcessor::*)(QWebSocketProtocol::CloseCode , const QString & )>(_a, &QWebSocketDataProcessor::errorEncountered, 7))
            return;
    }
}

const QMetaObject *QWebSocketDataProcessor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QWebSocketDataProcessor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23QWebSocketDataProcessorE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QWebSocketDataProcessor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void QWebSocketDataProcessor::pingReceived(const QByteArray & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QWebSocketDataProcessor::pongReceived(const QByteArray & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QWebSocketDataProcessor::closeReceived(QWebSocketProtocol::CloseCode _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void QWebSocketDataProcessor::textFrameReceived(const QString & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void QWebSocketDataProcessor::binaryFrameReceived(const QByteArray & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void QWebSocketDataProcessor::textMessageReceived(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void QWebSocketDataProcessor::binaryMessageReceived(const QByteArray & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void QWebSocketDataProcessor::errorEncountered(QWebSocketProtocol::CloseCode _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2);
}
QT_WARNING_POP
