/****************************************************************************
** Meta object code from reading C++ file 'qqmlwebsocketserver.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../qtwebsockets/src/imports/qmlwebsockets/qqmlwebsocketserver.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qqmlwebsocketserver.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19QQmlWebSocketServerE_t {};
} // unnamed namespace

template <> constexpr inline auto QQmlWebSocketServer::qt_create_metaobjectdata<qt_meta_tag_ZN19QQmlWebSocketServerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QQmlWebSocketServer",
        "QML.Element",
        "WebSocketServer",
        "QML.AddedInVersion",
        "256",
        "clientConnected",
        "",
        "QQmlWebSocket*",
        "webSocket",
        "errorStringChanged",
        "errorString",
        "urlChanged",
        "QUrl",
        "url",
        "portChanged",
        "port",
        "nameChanged",
        "name",
        "supportedSubprotocolsChanged",
        "supportedProtocols",
        "hostChanged",
        "host",
        "listenChanged",
        "listen",
        "acceptChanged",
        "accept",
        "supportedSubprotocols"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'clientConnected'
        QtMocHelpers::SignalData<void(QQmlWebSocket *)>(5, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'errorStringChanged'
        QtMocHelpers::SignalData<void(const QString &)>(9, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 10 },
        }}),
        // Signal 'urlChanged'
        QtMocHelpers::SignalData<void(const QUrl &)>(11, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Signal 'portChanged'
        QtMocHelpers::SignalData<void(int)>(14, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 15 },
        }}),
        // Signal 'nameChanged'
        QtMocHelpers::SignalData<void(const QString &)>(16, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 17 },
        }}),
        // Signal 'supportedSubprotocolsChanged'
        QtMocHelpers::SignalData<void(const QStringList &)>(18, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 19 },
        }}),
        // Signal 'hostChanged'
        QtMocHelpers::SignalData<void(const QString &)>(20, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 21 },
        }}),
        // Signal 'listenChanged'
        QtMocHelpers::SignalData<void(bool)>(22, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 23 },
        }}),
        // Signal 'acceptChanged'
        QtMocHelpers::SignalData<void(bool)>(24, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 25 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'url'
        QtMocHelpers::PropertyData<QUrl>(13, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 2),
        // property 'host'
        QtMocHelpers::PropertyData<QString>(21, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'port'
        QtMocHelpers::PropertyData<int>(15, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'name'
        QtMocHelpers::PropertyData<QString>(17, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'supportedSubprotocols'
        QtMocHelpers::PropertyData<QStringList>(26, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'errorString'
        QtMocHelpers::PropertyData<QString>(10, QMetaType::QString, QMC::DefaultPropertyFlags, 1),
        // property 'listen'
        QtMocHelpers::PropertyData<bool>(23, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'accept'
        QtMocHelpers::PropertyData<bool>(25, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<QQmlWebSocketServer, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QQmlWebSocketServer::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19QQmlWebSocketServerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19QQmlWebSocketServerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19QQmlWebSocketServerE_t>.metaTypes,
    nullptr
} };

void QQmlWebSocketServer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QQmlWebSocketServer *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->clientConnected((*reinterpret_cast<std::add_pointer_t<QQmlWebSocket*>>(_a[1]))); break;
        case 1: _t->errorStringChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->urlChanged((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 3: _t->portChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->nameChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->supportedSubprotocolsChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 6: _t->hostChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->listenChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->acceptChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QQmlWebSocketServer::*)(QQmlWebSocket * )>(_a, &QQmlWebSocketServer::clientConnected, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQmlWebSocketServer::*)(const QString & )>(_a, &QQmlWebSocketServer::errorStringChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQmlWebSocketServer::*)(const QUrl & )>(_a, &QQmlWebSocketServer::urlChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQmlWebSocketServer::*)(int )>(_a, &QQmlWebSocketServer::portChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQmlWebSocketServer::*)(const QString & )>(_a, &QQmlWebSocketServer::nameChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQmlWebSocketServer::*)(const QStringList & )>(_a, &QQmlWebSocketServer::supportedSubprotocolsChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQmlWebSocketServer::*)(const QString & )>(_a, &QQmlWebSocketServer::hostChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQmlWebSocketServer::*)(bool )>(_a, &QQmlWebSocketServer::listenChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQmlWebSocketServer::*)(bool )>(_a, &QQmlWebSocketServer::acceptChanged, 8))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QUrl*>(_v) = _t->url(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->host(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->port(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->name(); break;
        case 4: *reinterpret_cast<QStringList*>(_v) = _t->supportedSubprotocols(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->errorString(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->listen(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->accept(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setHost(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setPort(*reinterpret_cast<int*>(_v)); break;
        case 3: _t->setName(*reinterpret_cast<QString*>(_v)); break;
        case 4: _t->setSupportedSubprotocols(*reinterpret_cast<QStringList*>(_v)); break;
        case 6: _t->setListen(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->setAccept(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QQmlWebSocketServer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQmlWebSocketServer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19QQmlWebSocketServerE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    return QObject::qt_metacast(_clname);
}

int QQmlWebSocketServer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void QQmlWebSocketServer::clientConnected(QQmlWebSocket * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QQmlWebSocketServer::errorStringChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QQmlWebSocketServer::urlChanged(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QQmlWebSocketServer::portChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QQmlWebSocketServer::nameChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void QQmlWebSocketServer::supportedSubprotocolsChanged(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void QQmlWebSocketServer::hostChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void QQmlWebSocketServer::listenChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void QQmlWebSocketServer::acceptChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}
QT_WARNING_POP
