/****************************************************************************
** Meta object code from reading C++ file 'qqmlwebsocket.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../qtwebsockets/src/imports/qmlwebsockets/qqmlwebsocket.h"
#include <QtNetwork/QSslPreSharedKeyAuthenticator>
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qqmlwebsocket.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13QQmlWebSocketE_t {};
} // unnamed namespace

template <> constexpr inline auto QQmlWebSocket::qt_create_metaobjectdata<qt_meta_tag_ZN13QQmlWebSocketE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QQmlWebSocket",
        "QML.Element",
        "WebSocket",
        "QML.AddedInVersion",
        "256",
        "textMessageReceived",
        "",
        "message",
        "binaryMessageReceived",
        "statusChanged",
        "QQmlWebSocket::Status",
        "status",
        "activeChanged",
        "isActive",
        "errorStringChanged",
        "errorString",
        "urlChanged",
        "requestedSubprotocolsChanged",
        "negotiatedSubprotocolChanged",
        "pong",
        "elapsedTime",
        "payload",
        "onError",
        "QAbstractSocket::SocketError",
        "error",
        "onStateChanged",
        "QAbstractSocket::SocketState",
        "state",
        "sendTextMessage",
        "sendBinaryMessage",
        "ping",
        "url",
        "QUrl",
        "requestedSubprotocols",
        "Status",
        "active",
        "negotiatedSubprotocol",
        "Connecting",
        "Open",
        "Closing",
        "Closed",
        "Error"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'textMessageReceived'
        QtMocHelpers::SignalData<void(QString)>(5, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Signal 'binaryMessageReceived'
        QtMocHelpers::RevisionedSignalData<void(QByteArray)>(8, 6, QMC::AccessPublic, 0xff01, QMetaType::Void, {{
            { QMetaType::QByteArray, 7 },
        }}),
        // Signal 'statusChanged'
        QtMocHelpers::SignalData<void(QQmlWebSocket::Status)>(9, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'activeChanged'
        QtMocHelpers::SignalData<void(bool)>(12, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 13 },
        }}),
        // Signal 'errorStringChanged'
        QtMocHelpers::SignalData<void(QString)>(14, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 },
        }}),
        // Signal 'urlChanged'
        QtMocHelpers::SignalData<void()>(16, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestedSubprotocolsChanged'
        QtMocHelpers::SignalData<void()>(17, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'negotiatedSubprotocolChanged'
        QtMocHelpers::SignalData<void()>(18, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'pong'
        QtMocHelpers::RevisionedSignalData<void(quint64, const QByteArray &)>(19, 6, QMC::AccessPublic, 0x60a, QMetaType::Void, {{
            { QMetaType::ULongLong, 20 }, { QMetaType::QByteArray, 21 },
        }}),
        // Slot 'onError'
        QtMocHelpers::SlotData<void(QAbstractSocket::SocketError)>(22, 6, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 23, 24 },
        }}),
        // Slot 'onStateChanged'
        QtMocHelpers::SlotData<void(QAbstractSocket::SocketState)>(25, 6, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 26, 27 },
        }}),
        // Method 'sendTextMessage'
        QtMocHelpers::MethodData<qint64(const QString &)>(28, 6, QMC::AccessPublic, QMetaType::LongLong, {{
            { QMetaType::QString, 7 },
        }}),
        // Method 'sendBinaryMessage'
        QtMocHelpers::RevisionedMethodData<qint64(const QByteArray &)>(29, 6, QMC::AccessPublic, 0xff01, QMetaType::LongLong, {{
            { QMetaType::QByteArray, 7 },
        }}),
        // Method 'ping'
        QtMocHelpers::RevisionedMethodData<void()>(30, 6, QMC::AccessPublic, 0x60a, QMetaType::Void),
        // Method 'ping'
        QtMocHelpers::RevisionedMethodData<void(const QByteArray &)>(30, 6, QMC::AccessPublic, 0x60a, QMetaType::Void, {{
            { QMetaType::QByteArray, 21 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'url'
        QtMocHelpers::PropertyData<QUrl>(31, 0x80000000 | 32, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 5),
        // property 'requestedSubprotocols'
        QtMocHelpers::PropertyData<QStringList>(33, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'status'
        QtMocHelpers::PropertyData<enum Status>(11, 0x80000000 | 34, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 2),
        // property 'errorString'
        QtMocHelpers::PropertyData<QString>(15, QMetaType::QString, QMC::DefaultPropertyFlags, 4),
        // property 'active'
        QtMocHelpers::PropertyData<bool>(35, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'negotiatedSubprotocol'
        QtMocHelpers::PropertyData<QString>(36, QMetaType::QString, QMC::DefaultPropertyFlags, 7),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Status'
        QtMocHelpers::EnumData<enum Status>(34, 34, QMC::EnumFlags{}).add({
            {   37, Status::Connecting },
            {   38, Status::Open },
            {   39, Status::Closing },
            {   40, Status::Closed },
            {   41, Status::Error },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<QQmlWebSocket, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QQmlWebSocket::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13QQmlWebSocketE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13QQmlWebSocketE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13QQmlWebSocketE_t>.metaTypes,
    nullptr
} };

void QQmlWebSocket::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QQmlWebSocket *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->textMessageReceived((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->binaryMessageReceived((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 2: _t->statusChanged((*reinterpret_cast<std::add_pointer_t<QQmlWebSocket::Status>>(_a[1]))); break;
        case 3: _t->activeChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->errorStringChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->urlChanged(); break;
        case 6: _t->requestedSubprotocolsChanged(); break;
        case 7: _t->negotiatedSubprotocolChanged(); break;
        case 8: _t->pong((*reinterpret_cast<std::add_pointer_t<quint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 9: _t->onError((*reinterpret_cast<std::add_pointer_t<QAbstractSocket::SocketError>>(_a[1]))); break;
        case 10: _t->onStateChanged((*reinterpret_cast<std::add_pointer_t<QAbstractSocket::SocketState>>(_a[1]))); break;
        case 11: { qint64 _r = _t->sendTextMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<qint64*>(_a[0]) = std::move(_r); }  break;
        case 12: { qint64 _r = _t->sendBinaryMessage((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast<qint64*>(_a[0]) = std::move(_r); }  break;
        case 13: _t->ping(); break;
        case 14: _t->ping((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstractSocket::SocketError >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstractSocket::SocketState >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QQmlWebSocket::*)(QString )>(_a, &QQmlWebSocket::textMessageReceived, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQmlWebSocket::*)(QByteArray )>(_a, &QQmlWebSocket::binaryMessageReceived, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQmlWebSocket::*)(QQmlWebSocket::Status )>(_a, &QQmlWebSocket::statusChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQmlWebSocket::*)(bool )>(_a, &QQmlWebSocket::activeChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQmlWebSocket::*)(QString )>(_a, &QQmlWebSocket::errorStringChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQmlWebSocket::*)()>(_a, &QQmlWebSocket::urlChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQmlWebSocket::*)()>(_a, &QQmlWebSocket::requestedSubprotocolsChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQmlWebSocket::*)()>(_a, &QQmlWebSocket::negotiatedSubprotocolChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQmlWebSocket::*)(quint64 , const QByteArray & )>(_a, &QQmlWebSocket::pong, 8))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QUrl*>(_v) = _t->url(); break;
        case 1: *reinterpret_cast<QStringList*>(_v) = _t->requestedSubprotocols(); break;
        case 2: *reinterpret_cast<enum Status*>(_v) = _t->status(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->errorString(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->isActive(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->negotiatedSubprotocol(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setUrl(*reinterpret_cast<QUrl*>(_v)); break;
        case 1: _t->setRequestedSubprotocols(*reinterpret_cast<QStringList*>(_v)); break;
        case 4: _t->setActive(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QQmlWebSocket::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQmlWebSocket::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13QQmlWebSocketE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    return QObject::qt_metacast(_clname);
}

int QQmlWebSocket::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void QQmlWebSocket::textMessageReceived(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QQmlWebSocket::binaryMessageReceived(QByteArray _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QQmlWebSocket::statusChanged(QQmlWebSocket::Status _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QQmlWebSocket::activeChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QQmlWebSocket::errorStringChanged(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void QQmlWebSocket::urlChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QQmlWebSocket::requestedSubprotocolsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QQmlWebSocket::negotiatedSubprotocolChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QQmlWebSocket::pong(quint64 _t1, const QByteArray & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1, _t2);
}
QT_WARNING_POP
